package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.*;

import java.time.Duration;

public class AkkaMainSystem extends AbstractBehavior<AkkaMainSystem.Message> {

    private final TimerScheduler<Message> timers;
    private ActorRef<RelayActor.Message> alice;
    private ActorRef<RelayActor.Message> bob;
    private ActorRef<RelayActor.Message> charlie;

    public interface Message {    }
    public static class Create implements Message {}
    public enum sendTimeOut implements Message { INSTANCE }

    public static Behavior<Message> create() {
        return Behaviors.setup(context -> Behaviors.withTimers(timers -> new AkkaMainSystem(context, timers)));
    }

    private AkkaMainSystem(ActorContext<Message> context, TimerScheduler<Message> timers) {
        super(context);
        this.timers = timers;
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(Create.class, this::onCreate)
                .onMessage(sendTimeOut.class, this::onSendTimeOut)
                .build();
    }

    private Behavior<Message> onSendTimeOut(sendTimeOut msg) {
        alice.tell(RelayActor.TimeOut.INSTANCE);
        bob.tell(RelayActor.TimeOut.INSTANCE);
        charlie.tell(RelayActor.TimeOut.INSTANCE);
        return this;
    }

    private Behavior<Message> onCreate(Create command) {
        //#create-actors
        alice = this.getContext().spawn(RelayActor.create(), "alice");
        bob = this.getContext().spawn(RelayActor.create(), "bob");
        charlie = this.getContext().spawn(RelayActor.create(), "charlie");
        //#create-actors

        alice.tell(new RelayActor.Init(charlie, bob));
        bob.tell(new RelayActor.Init(alice, charlie));
        charlie.tell(new RelayActor.Init(bob, alice));

        timers.startSingleTimer(sendTimeOut.INSTANCE, Duration.ofSeconds(60));

        alice.tell(RelayActor.ClockWise.MSG);
        alice.tell(RelayActor.CounterClockWise.MSG);
        return this;
    }
}
