package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.javadsl.TimerScheduler;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

import java.time.Duration;


public class RelayActor extends AbstractBehavior<RelayActor.Message> {

    private ActorRef<Message> next;
    private ActorRef<Message> prev;

    public interface Message {};

    public record Init(ActorRef<RelayActor.Message> prev, ActorRef<RelayActor.Message> next) implements Message {}
    public enum ClockWise implements Message { MSG }
    public enum CounterClockWise implements Message { MSG }
    public enum sendClockWise implements Message { INSTANCE }
    public enum sendCounterClockWise implements Message { INSTANCE }
    public enum TimeOut implements Message {INSTANCE}
    public static Behavior<Message> create() {
        return Behaviors.setup(context -> Behaviors.withTimers(timers -> new RelayActor(context, timers)));
    }

    private final TimerScheduler<RelayActor.Message> timers;

    private RelayActor(ActorContext<Message> context, TimerScheduler<RelayActor.Message> timers) {
        super(context);
        this.timers = timers;
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(Init.class, this::onInit)
                .onMessage(ClockWise.class, this::onClockWise)
                .onMessage(CounterClockWise.class, this::onCounterClockWise)
                .onMessage(sendClockWise.class, this::onSendClockWise)
                .onMessage(sendCounterClockWise.class, this::onSendCounterClockWise)
                .onMessage(TimeOut.class, this::onTimeOut)
                .build();
    }

    private Behavior<Message> onInit(Init msg) {
        this.next = msg.next;
        this.prev = msg.prev;
        return this;
    }
    private Behavior<Message> onClockWise(ClockWise msg) {
        this.getContext().getLog().info("Got Clockwise");
        this.timers.startSingleTimer(sendClockWise.INSTANCE, Duration.ofSeconds(3));
        return this;
    }

    private Behavior<Message> onSendClockWise(sendClockWise msg) {
        this.next.tell(ClockWise.MSG);
        return this;
    }

    private Behavior<Message> onCounterClockWise(CounterClockWise msg) {
        this.getContext().getLog().info("Got CounterClockwise");
        this.timers.startSingleTimer(sendCounterClockWise.INSTANCE, Duration.ofSeconds(5));
        return this;
    }

    private Behavior<Message> onSendCounterClockWise(sendCounterClockWise msg) {
        this.prev.tell(CounterClockWise.MSG);
        return this;
    }

    private Behavior<Message> onTimeOut(TimeOut msg) {
        this.getContext().getLog().info("TimeOut");
        return Behaviors.stopped();
    }
}
